using System;
using System.Diagnostics;
using System.ServiceModel;

using BA.ECommerce.Contracts;

namespace BA.ECommerce.Client
{
	class Program
	{
		static void Main(string[] args)
		{
			//testPlaceOrder();
			//testCheckOrderStatus();
			testPlaceOrderWithApproval();

			Console.WriteLine("Press any key to stop the client");
			Console.Read();
		}

		private static void testPlaceOrder()
		{
			//string address = "net.tcp://localhost:9000/BA.ECommerce.Purchasing";
			string address = "net.tcp://localhost:9999/BA.ECommerce.BizTalk/Purchasing";

			IPurchasing proxy = ChannelFactory<IPurchasing>.CreateChannel(
				new NetTcpBinding(), 
				new EndpointAddress(address));
			using (proxy as IDisposable)
			{
				try
				{
					PurchaseOrder po = getPurchaseOrder();
					Console.WriteLine("Placing order\n{0}\n", po.PurchaseOrderID);
					proxy.PlaceOrder(po);
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Trace.WriteLine(ex);
					Console.Read();
				}
			}
		}

		private static void testCheckOrderStatus()
		{
			string address = "net.tcp://localhost:9090/BA.ECommerce.BizTalk.CheckOrderStatus";
			NetTcpBinding binding = new NetTcpBinding();

			IPurchasing proxy = ChannelFactory<IPurchasing>.CreateChannel(
				binding,
				new EndpointAddress(address));
			using (proxy as IDisposable)
			{
				try
				{
					PurchaseOrder po = getPurchaseOrder();
					Console.WriteLine("Checking status of \n{0}\n", po.PurchaseOrderID);
					CheckOrderStatus request = new CheckOrderStatus(po);
					CheckOrderStatusResponse response = proxy.CheckOrderStatus(request);
					Console.WriteLine("Status of order {0} is {1}", response.PurchaseOrderStatus.PurchaseOrderID, response.PurchaseOrderStatus.Status);
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Trace.WriteLine(ex);
					Console.Read();
				}
			}
		}

		private static void testPlaceOrderWithApproval()
		{

			#region Send PO

			// string address = "net.tcp://localhost:9000/BA.ECommerce.Purchasing";
			string address = "net.tcp://localhost:9898/BA.ECommerce.PlaceOrder";
			NetTcpBinding binding = new NetTcpBinding();

			//BasicHttpBinding binding = new BasicHttpBinding();
			//string address = "http://localhost:5000/BA.ECommerce.Purchasing";

			IPurchasing proxy = ChannelFactory<IPurchasing>.CreateChannel(
				binding,
				new EndpointAddress(address));
			using (proxy as IDisposable)
			{
				try
				{
					PurchaseOrder po;
					for (int i = 0; i < 3; ++i)
					{
						po = getPurchaseOrder();
						Console.WriteLine("Placing order\n{0}\n", po.PurchaseOrderID);
						proxy.PlaceOrderWithApproval(new PurchaseOrderMessage(po));
					}
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Trace.WriteLine(ex);
					Console.Read();
				}
			}

			#endregion

			#region Listen for PO Status

			ServiceHost host = new ServiceHost(
				typeof(PurchasingClient),
				new Uri("net.tcp://localhost:7777/"));

			host.AddServiceEndpoint(
				typeof(IPurchasingClient),
				new NetTcpBinding(),
				"BA.ECommerce.ReceiveOrderStatus");

			using (host)
			{
				try
				{
					host.Open();
					Console.WriteLine("\nPress any key to stop the client service.\n");
					Console.Read();
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Trace.WriteLine(ex);
					Console.Read();
				}
			}

			#endregion
		}

		private static void testCancelPurchaseOrder()
		{
			// string address = "net.tcp://localhost:9000/BA.ECommerce.Purchasing";
			string address = "net.tcp://localhost:9595/BA.ECommerce.CancelOrder";
			NetTcpBinding binding = new NetTcpBinding();

			IPurchasing proxy = ChannelFactory<IPurchasing>.CreateChannel(
				binding,
				new EndpointAddress(address));
			using (proxy as IDisposable)
			{
				try
				{
					PurchaseOrder po = getPurchaseOrder();
					Console.WriteLine("Cancelling order \n{0}\n", po.PurchaseOrderID);
					proxy.CancelOrder(new CancelOrder(po));
					CancelOrder request = new CancelOrder(po);
					proxy.CancelOrder(request);
					// proxy.CancelOrder(po);
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex);
					Trace.WriteLine(ex);
					Console.Read();
				}
			}
		}

		private static PurchaseOrder getPurchaseOrder()
		{
			PurchaseOrder po = new PurchaseOrder();
			po.PurchaseOrderID = Guid.NewGuid().ToString();
			po.CustomerNumber = "ACME1234";
			po.OrderDate = DateTime.Today;
			po.DeliveryDate = DateTime.Today.AddDays(5);
			po.OrderDetails = "Twelve boxes of ACME Earthquake pills";
			return po;
		}

	}
}
