using System;
using System.Diagnostics;
using System.ServiceModel;

using BA.ECommerce.Contracts;

namespace BA.ECommerce.Service
{
	[ServiceBehavior(IncludeExceptionDetailInFaults=true)]
	class Purchasing : IPurchasing
	{
		#region IPurchasing Members

		public void PlaceOrder(PurchaseOrder po)
		{
			if (po == null)
			{
				Console.WriteLine("PurchaseOrder is null.");
			}
			else
			{
				Console.WriteLine("\nReceived purchase order\n{0}\ncustomer {1}\n{2}",
					po.PurchaseOrderID, po.CustomerNumber, po.OrderDetails);
				Trace.WriteLine(String.Format("Received purchase order {0} from customer {1}: {2}\n",
					po.PurchaseOrderID, po.CustomerNumber, po.OrderDetails));
			}
		}

		public void PlaceOrderWithApproval(PurchaseOrderMessage request)
		{
			PlaceOrder(request.PurchaseOrder);
		}

		public CheckOrderStatusResponse CheckOrderStatus(CheckOrderStatus request)
		{
			PurchaseOrder po = request.PurchaseOrder;
			PurchaseOrderStatus status = new PurchaseOrderStatus();
			status.CustomerNumber = po.CustomerNumber;
			status.PurchaseOrderID = po.PurchaseOrderID;
			status.Status = PurchaseOrderStatus.OrderStatus.PendingApproval;

			Console.WriteLine("\nStatus requested for order {0} from customer {1}: {2}",
				status.PurchaseOrderID, status.CustomerNumber, status.Status);
			Trace.WriteLine(String.Format("Status requested for order {0} from customer {1}: {2}",
				status.PurchaseOrderID, status.CustomerNumber, status.Status));

			return new CheckOrderStatusResponse(status);
		}

		public void CancelOrder(CancelOrder request)
		{
			Console.WriteLine("\nCancelled order {0} from customer {1}",
				request.PurchaseOrder.PurchaseOrderID, request.PurchaseOrder.CustomerNumber);
			Trace.WriteLine(String.Format("Cancelled order {0} from customer {1}",
				request.PurchaseOrder.PurchaseOrderID, request.PurchaseOrder.CustomerNumber));
		}

		//public void CancelOrder(PurchaseOrder po)
		//{
		//    Console.WriteLine("Cancelled order {0} from customer {1}",
		//        po.PurchaseOrderID, po.CustomerNumber);
		//    Trace.WriteLine(String.Format("Cancelled order {0} from customer {1}",
		//        po.PurchaseOrderID, po.CustomerNumber));
		//}

		#endregion
	}
}
