using System;

using Contracts;

namespace Services
{
	public abstract class StepBase : IEmploymentProcessingStep
	{
		protected abstract void ExecuteStep(JobApplicant applicant);

		#region IEmploymentProcessingStep Members

		public void DoStep(EmploymentProcessingStepMessage stepRequest)
		{
			#region Execute the current step

			ExecuteStep(stepRequest.ProcessingState.Applicant);

			#endregion

			#region Set the next step

			if (stepRequest.ProcessingState.Steps != null && stepRequest.ProcessingState.Steps.Length > 0)
			{
				int currentStepIndex = Array.FindIndex<string>(stepRequest.ProcessingState.Steps,
					delegate(string s) { return s == stepRequest.ProcessingState.CurrentStep; });
				if (currentStepIndex > -1 && currentStepIndex < (stepRequest.ProcessingState.Steps.Length - 1))
				{
					stepRequest.ProcessingState.CurrentStep = stepRequest.ProcessingState.Steps[++currentStepIndex];
				}
			}

			#endregion

			#region Call the next step

			CallNextStep.Call(stepRequest);

			#endregion
		}

		#endregion
	}
}
