using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.ServiceModel;

using Contracts;
using Services;

namespace ServicesHost
{
	class Program
	{
		static void Main(string[] args)
		{
			List<ServiceHost> hosts = new List<ServiceHost>();
			hosts.Add(makeServiceHost(typeof(InitializeStepList), "InitializeStepList"));
			hosts.Add(makeServiceHost(typeof(CheckCriminalRecord), "CheckCriminalRecord"));
			hosts.Add(makeServiceHost(typeof(CheckCreditHistory), "CheckCreditHistory"));
			hosts.Add(makeServiceHost(typeof(CheckImmigrationStatus), "CheckImmigrationStatus"));
			hosts.Add(makeServiceHost(typeof(ConductJobInterview), "ConductJobInterview"));
			hosts.Add(makeServiceHost(typeof(MakeHiringDecision), "MakeHiringDecision"));
			hosts.Add(makeServiceHost(typeof(TerminationStep), "TerminationStep"));

			try
			{
				foreach (ServiceHost host in hosts)
				{
					host.Open();
				}
				Console.WriteLine("Press any key to stop the services.\n");
				Console.Read();
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex);
				Trace.WriteLine(ex);
				Console.Read();
			}
			finally
			{
				foreach (ServiceHost host in hosts)
				{
					host.Close();
				}
			}

		}

		private static ServiceHost makeServiceHost(Type serviceType, string serviceName)
		{
			Uri uri = new Uri("net.tcp://localhost:3000/");
			NetTcpBinding binding = new NetTcpBinding();

			ServiceHost host = new ServiceHost(serviceType, uri);
			host.AddServiceEndpoint(typeof(IEmploymentProcessingStep), binding, serviceName);
			return host;
		}
	}
}
